#Script to analyze the No of cells per organoid in P3E28 

require(tidyverse)
require(ggplot2)
require(viridis)

#Set wd
setwd("YOUR DIR - SET TO SAME AS FOR OUTPUT LEVEL")

arg <- "./E28"

col <- c(1,2,3)
col <- as.data.frame(col) %>%
  mutate(col2 = c("a", "e", "f")) %>%
  group_by(col) %>%
  nest()

#Helper function to read individual replicates for central organoids and nuclei/organoid data tables
f_load_cent <- function(tbl){
  x <- tbl$col2
  y <- read_tsv(paste0(arg, x, " Rescan/Objects_Population - 3) Organoid_P_Central.txt"),  skip = 9, col_names = T) %>%
    mutate(ident = paste0(Row,"_", Column)) %>%
    select(ident, Field, object = "Object No", 
           org_volume = "3) Organoid_P_Central - 4) Organoid_R_Central_Morphology Volume [m]", 
           org_sarea = "3) Organoid_P_Central - 4) Organoid_R_Central_Morphology Surface Area [m]",
           org_sphericity = "3) Organoid_P_Central - 4) Organoid_R_Central_Morphology Sphericity",
           org_innerr = "3) Organoid_P_Central - 4) Organoid_R_Central_Morphology Inner Sphere Radius [m]")
  return(y)
}

f_load_nuc <- function(tbl){
  x <- tbl$col2
  y <- read_tsv(paste0(arg, x, " Rescan/Objects_Population - 7) Nuclei.txt"),  skip = 9, col_names = T) %>%
    mutate(ident = paste0(Row,"_", Column)) %>%
    select(ident, Field, object = "Object No", 
           org = "7) Nuclei - ROI No", 
           volume = "7) Nuclei - 8) Nuclei_Morphology Volume [m]", 
           sphericity = "7) Nuclei - 8) Nuclei_Morphology Sphericity")
  return(y)
}


#Reads all replicates in a listed table environment 
f_central <- col %>%
  mutate(data2 = map(.x = data, .f = f_load_cent)) %>%
  unnest(data) %>%
  unnest(data2) %>%
  filter(!(object > 1))

#Load nuclei data and make organoid to 1 as half-spheres can be separated falsely into multiple organoids
f_nuclei <- col %>%
  mutate(data2 = map(.x = data, .f = f_load_nuc)) %>%
  unnest(data) %>%
  unnest(data2) %>%
  mutate(org = 1)

#Join organoids and nuclei/organoid tables
f_data <- left_join(f_central, f_nuclei, by = c("col", "col2", "ident", "Field")) %>%
  filter(!is.na(object.y))

#Change columns in replicate a according to exp conditions (Replicate a plate setup was slightly different than e and f)
a_C <- f_data %>% filter(col2 == "a" & ident == "2_4") %>% ungroup() 
a_FC <- f_data %>% filter(col2 == "a" & ident == "3_5") %>% ungroup() %>% mutate(ident = "3_2") 
a_BC <- f_data %>% filter(col2 == "a" & ident == "3_2") %>% ungroup() %>% mutate(ident = "3_3") 
a_FBC <- f_data %>% filter(col2 == "a" & ident == "3_3") %>% ungroup() %>% mutate(ident = "3_4") 
a_PegO <- f_data %>% filter(col2 == "a" & ident == "3_4") %>% ungroup() %>% mutate(ident = "3_5") 

a_full <- rbind(a_C, a_FC, a_BC, a_FBC, a_PegO) 
f_data <- f_data %>% filter(col2 != "a") %>% as.data.frame()

f_data <- rbind(a_full, f_data) %>%
  group_by(col, col2, ident, Field, object.x) %>%
  nest() 


#helper function to get length(ncol(x)) for each organoid (number of cells for each organoid)
f_nrow <- function(tbl){
  x <- NROW(tbl)
  return(x)
}

#Mappr function to get nrow for each organoid which corresponds to the individual nuclei/organoid
f_data <- f_data %>%
  mutate(cell_org = as.numeric(map(.x = data, .f = f_nrow)),
         cell_org2 = as.numeric(cell_org)*2)

#Get data only for gels containing the GFOGER peptide ("C")
C_gels <- f_data %>% select(-data) %>% ungroup() %>% filter(ident =="2_4" | 
                          ident =="3_2" | 
                          ident =="3_3"| 
                          ident =="3_4"| 
                          ident =="3_5") %>%
  mutate(ident = str_replace(ident, pattern = "2_4", "1"), #1 - C
         ident = str_replace(ident, pattern = "3_2", "2"), #2 - FC
         ident = str_replace(ident, pattern = "3_3", "3"), #3 - BC
         ident = str_replace(ident, pattern = "3_5", "4"), #4 - CBF-0.5
         ident = str_replace(ident, pattern = "3_4", "5")) #5 - CBF-1.0

#Mapper function to calculate mean for violin plot in dens plot SF12b
f_mean <- function(tbl){
  x <- as.numeric(mean(tbl$cell_org))
  return(x)
}

grp_mean <- C_gels %>% 
  group_by(ident) %>% 
  nest() %>% 
  mutate(mean = map(.x = data, .f = f_mean)) %>%
  select(-data) %>%
  ungroup()

grp_mean <- grp_mean %>% mutate(mean = as.numeric(mean))

#Plot the results 
out_1 <- ggplot(C_gels %>% arrange(ident), mapping = aes(x = cell_org, fill = ident, color = ident))+
  geom_density(stat = "density", alpha = 0.25)+
  geom_vline(data = grp_mean, aes(xintercept = mean, color = ident), linetype = 3)+
  facet_wrap(vars(ident), ncol = 1, scales = "free_y")+
  scale_fill_manual(values = c("#05BE78", "#B4F8DF", "#FFC0E0","#F96495" , "#F71480")) +
  scale_color_manual(values = c("#05BE78", "#B4F8DF", "#FFC0E0","#F96495" , "#F71480"))+
  theme_classic()+
  theme(
    strip.background = element_blank(),
    strip.text.x = element_blank()
  )+
  xlab("cells/organoid")+
  ylab("Density")

ggsave("./SF12b_HCS_nuclei.pdf", out_1, device = "pdf", width = 10, height = 10, unit = "in")

#Calculate mean number of cells per organoid and paste data in a table
C_gels <- f_data %>% ungroup() %>% filter(ident =="2_4" | 
                                                              ident =="3_2" | 
                                                              ident =="3_3"| 
                                                              ident =="3_4"| 
                                                              ident =="3_5") %>%
  mutate(ident = str_replace(ident, pattern = "2_4", "C"),
         ident = str_replace(ident, pattern = "3_2", "CF"),
         ident = str_replace(ident, pattern = "3_3", "CB"),
         ident = str_replace(ident, pattern = "3_5", "CBF-0.5"),
         ident = str_replace(ident, pattern = "3_4", "CBF-1.0")) %>%
  unnest(cols = c(data)) %>%
  ungroup() %>%
  select(-col2, -object.x, -cell_org2)

write.csv(C_gels, file = "./SF12b_HCS_nuclei_output.csv", row.names = F)

